<?php
/*
Plugin Name: External DB authentication
Plugin URI: http://www.ploofle.com/tag/ext_db_auth/
Description: Used to externally authenticate WP users with an existing user DB.
Version: 2.5
Author: Charlene Barina
Author URI: http://www.ploofle.com

    Copyright 2007  Charlene Barina  (email : cbarina@u.washington.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


//This plugin first checks at authentication time whether a user exists in WP
//with that login.  If not, and the password matches with one in the external
//db, it will use this info to create a new user account with idential login/pw
//in WP.  If a user exists with that account in WP, the password/user data will be
//resynched from the external db and login will continue.  There isn't much point
//to updating user profile info in WP, though, as if there is any additional data
//from the external db, it will be overwritten in the next sync.

add_action('activate_ext_db_auth.php', 'ext_db_auth_install');
add_action('admin_menu', 'ext_db_auth_add_menu');
add_action( 'wp_authenticate', 'ext_db_auth_check_login', 1, 2 );
add_filter('login_errors','ext_db_auth_login_errors');

//backwords compatability with php < 5 for htmlspecialchars_decode
if ( !function_exists('htmlspecialchars_decode') )
{
    function htmlspecialchars_decode($text)
    {
        return strtr($text, array_flip(get_html_translation_table(HTML_SPECIALCHARS)));
    }
}

//inserts custom error msg, if it exists, on invalid logins that aren't due to blank fields
function ext_db_auth_login_errors($error) {
	if (stristr($error,"Invalid") || stristr($error,"Incorrect")) {
		if (get_option('ext_db_error_msg') != "")
			$error = get_option('ext_db_error_msg');
	}
	return $error;
}

//install fxn
function ext_db_auth_install() {
	add_option('ext_host',"","External database hostname");
	add_option('ext_db',"","External database name");
	add_option('ext_db_user',"","External database username");
	add_option('ext_db_pw',"","External database password");
	add_option('ext_db_table',"","External database table for authentication");
	add_option('ext_db_namefield',"","External database field for username");
	add_option('ext_db_pwfield',"","External database field for password");
	add_option('ext_db_first_name',"");
    add_option('ext_db_last_name',"");
    add_option('ext_db_user_url',"");
    add_option('ext_db_user_email',"");
    add_option('ext_db_description',"");
    add_option('ext_db_aim',"");
    add_option('ext_db_yim',"");
    add_option('ext_db_jabber',"");
    add_option('ext_db_enc',"","Type of encoding for external db (default SHA1? or MD5?)");
	add_option('ext_db_error_msg',"","Custom login error message");   
    add_option('ext_db_other_enc',"$password2 = $password");
}
                             
//page for config menu
function ext_db_auth_add_menu() {
	add_options_page("External DB settings", "External DB settings", 10, "ext_db_auth.php","ext_db_auth_display_options");
}

//updated option code
function ext_db_auth_display_options() {
    ?>
    <div class="wrap">
    <form method="post" action="options.php">
        <?php wp_nonce_field('update-options'); ?>        
        <h2>External database settings</h2>        
        <h3>Connection settings</h3>
        <p>All of these fields are required.  If you make a mistake, it will lock you out of your Wordpress installation, and you'll have to delete or rename the plugin file to regain access!</p>
        <table class="form-table">
        <tr>
            <th><label>External database host:</label></th><td><input type="text" name="ext_host" value="<?php echo get_option('ext_host'); ?>" /> <br/><strong style="color:red;">required</strong>; (often localhost) <td>
        </tr>
        <tr>
            <th><label>External database name:</label></th><td><input type="text" name="ext_db" value="<?php echo get_option('ext_db'); ?>" /><br/><strong style="color:red;">required</strong><td>
        </tr>
        <tr>
            <th><label>External database username:</label></th><td><input type="text" name="ext_db_user" value="<?php echo get_option('ext_db_user'); ?>" /><br/><strong style="color:red;">required</strong>; (recommend select privileges only)<td>
        </tr>
        <tr>
            <th><label>External database password:</label></th><td><input type="password" name="ext_db_pw" value="<?php echo get_option('ext_db_pw'); ?>" /><br/><strong style="color:red;">required</strong><td>
        </tr>
        <tr>
            <th><label>Table containing users:</label></th><td><input type="text" name="ext_db_table" value="<?php echo get_option('ext_db_table'); ?>" /><br/><strong style="color:red;">required</strong><td>
        </tr>
        
        </table>
        
        <h3>Field matching settings</h3>
        <p>Username, password, and password hash type have to be set at the minimum.</p>
        <table class="form-table">
        <tr>
            <th><label>Username field:</label></th><td><input type="text" name="ext_db_namefield" value="<?php echo get_option('ext_db_namefield'); ?>" /><br/><strong style="color:red;">required</strong><td>
        </tr>
        <tr>
            <th><label>User password field:</label></th><td><input type="text" name="ext_db_pwfield" value="<?php echo get_option('ext_db_pwfield'); ?>" /><br/><strong style="color:red;">required</strong><td>
        </tr>
        <tr>
            <th>Type of encryption for password:</th>
                <td><select name="ext_db_enc">
                <?php 
                    switch(get_option('ext_db_enc')) {
                    case "SHA1" :
                        echo '<option selected="selected">SHA1</option><option>MD5</option><option>Other</option>';
                        break;
                    case "MD5" :
                        echo '<option>SHA1</option><option selected="selected">MD5</option><option>Other</option>';
                        break;                
                    case "Other" :
                        echo '<option>SHA1</option><option  selected="selected">MD5</option><option selected="selected">Other</option>';
                        break;                                        
                    default :
                        echo '<option selected="selected">SHA1</option><option>MD5</option><option>Other</option>';
                        break;
                    }
                ?>
            </select><br/><strong style="color:red;">required</strong>; (using "Other" requires you to enter PHP code below!)</td>            
        </tr>
        <tr>
            <th><label>Other encryption:</label></th><td><input type="text" name="ext_db_other_enc" size="100" value="<?php echo get_option('ext_db_other_enc'); ?>" /><br/>Enter code here; only will run if "Other" is selected. Variable you need to set is $password2. See source code for other variable names.<td>
        </tr>
        <tr>
            <th><label>User first name field:</label></th><td><input type="text" name="ext_db_first_name" value="<?php echo get_option('ext_db_first_name'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User last name field:</label></th><td><input type="text" name="ext_db_last_name" value="<?php echo get_option('ext_db_last_name'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User homepage field:</label></th><td><input type="text" name="ext_db_user_url" value="<?php echo get_option('ext_db_user_url'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User email field:</label></th><td><input type="text" name="ext_db_user_email" value="<?php echo get_option('ext_db_user_email'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User bio/description field:</label></th><td><input type="text" name="ext_db_description" value="<?php echo get_option('ext_db_description'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User AIM screen name field:</label></th><td><input type="text" name="ext_db_aim" value="<?php echo get_option('ext_db_aim'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User YIM screen name field:</label></th><td><input type="text" name="ext_db_yim" value="<?php echo get_option('ext_db_yim'); ?>" /><td>
        </tr>
        <tr>
            <th><label>User JABBER screen name field:</label></th><td><input type="text" name="ext_db_jabber" value="<?php echo get_option('ext_db_jabber'); ?>" /><td>
        </tr>
        </table>
        <h3>Other</h3>
        <table class="form-table">
        <tr valign="top">
                <th scope="row">Custom login error message:</th>
                <td><input name="ext_db_error_msg" type="text" size="120" value="<?php echo htmlspecialchars(get_option('ext_db_error_msg'));?>"/>
                    <br />(used if invalid pw or invalid username. leave blank if you don't want to use this.)</td>
            </tr>        
            </table>
        <input type="hidden" name="action" value="update" />
        <input type="hidden" name="page_options" value="ext_host,ext_db,ext_db_user,ext_db_pw,ext_db_table,ext_db_namefield,ext_db_pwfield,ext_db_first_name,ext_db_last_name,ext_db_user_url,ext_db_user_email,ext_db_description,ext_db_aim,ext_db_yim,ext_db_jabber,ext_db_enc,ext_db_error_msg,ext_db_other_enc" />
        <p class="submit"><input type="submit" name="Submit" value="<?php _e('Save Changes') ?>" /></p>
    </form>
    </div>
    <?php
}


//actual meat of plugin
function ext_db_auth_check_login($username,$password) {
	require_once('./wp-includes/registration.php');
   
	//if on same host have to use resource id to make sure you don't lose the wp db connection    
    $ext = mysql_connect(get_option('ext_host'), get_option('ext_db_user'), get_option('ext_db_pw'),true) or die(mysql_error());
    mysql_select_db(get_option('ext_db'),$ext) or die(mysql_error());
	switch(get_option('ext_db_enc')) {
		case "SHA1" :
			$password2 = sha1($password);
			break;
		case "MD5" :
			$password2 = md5($password);
			break;			
        case "Other" :             //right now defaulting to plaintext.  People can change code here for their own special hash
            eval(get_option('ext_db_other_enc'));
            break;
	}
    
    //first check to see if login exists in external db
	$query = "SELECT " . get_option('ext_db_namefield') . ", ". get_option('ext_db_pwfield') ." FROM " . get_option('ext_db_table') . " WHERE ".get_option('ext_db_namefield')." = '$username' limit 1";
    $result = mysql_query($query,$ext) or die(mysql_error());
    if (mysql_num_rows($result)) {
        //then check to see if pw matches and get other fields...
        $sqlfields[] = get_option('ext_db_first_name');
        $sqlfields[] = get_option('ext_db_last_name');
        $sqlfields[] = get_option('ext_db_user_url');
        $sqlfields[] = get_option('ext_db_user_email');
        $sqlfields[] = get_option('ext_db_description');
        $sqlfields[] = get_option('ext_db_aim');
        $sqlfields[] = get_option('ext_db_yim');
        $sqlfields[] = get_option('ext_db_jabber');    
        foreach($sqlfields as $key=>$value) {
            if ($value == "") unset($sqlfields[$key]);
        }
        $sqlfields2 = implode(", ",$sqlfields);
        $query = "SELECT $sqlfields2 FROM " . get_option('ext_db_table') . " WHERE ".get_option('ext_db_namefield')." = '$username' AND ".get_option('ext_db_pwfield')." = '$password2' limit 1";        
        //if matches, do nothing else.  If doesn't match, insert or update information.
        $result = mysql_query($query,$ext) or die(mysql_error());    
        if (mysql_num_rows($result)) {    //create/update wp account from external database if login/pw exact match exists in that db
            $userarray['user_login'] = $username;
            $userarray['user_pass'] = $password;            
            $extfields = mysql_fetch_row($result);            
            $userarray['first_name'] = $extfields[0];
            $userarray['last_name'] = $extfields[1];        
            $userarray['user_url'] = $extfields[2];
            $userarray['user_email'] = $extfields[3];
            $userarray['description'] = $extfields[4];
            $userarray['aim'] = $extfields[5];
            $userarray['yim'] = $extfields[6];
            $userarray['jabber'] = $extfields[7];
            $userarray['display_name'] = $extfields[0]." ".$extfields[1];            
            mysql_close($ext);
            
            //looks like wp functions clean up data before entry, so I'm not going to try to clean out fields beforehand.
            if ($id = username_exists($username)) {   //just do an update
                $userarray['ID'] = $id;
                wp_update_user($userarray);
            }
            else wp_insert_user($userarray);          //otherwise create
        }
        else {  //this is a poor way of dealing with unsynchronized pw, but it'll work for now.
            wp_redirect('./wp-login.php');
            exit();                                                                                                                                                                                                
        }
    }
}
?>